﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            /****************************注意例子中的使用方法****************
            * delegate TResult Func<TResult>();  无参,但是返回值为TResult类型
            * delegate void Action();  无参,无返回值
            * delegate bool Predicate<T>(T arg);  有一个参数arg,返回bool类型
            * 需要注意,与WinForm中不同的是,WPF中需要利用Control.Dispatcher.Invoke来实现,其他类似.
            * **************************************************************/
        }

        #region 利用Func实现线程和界面交互
        private void AlternationUsingFunc(object text)
        {
            //无参数,但是返回值为bool类型
            button1.Dispatcher.Invoke(new Func<bool>(delegate()
            {
                button1.Content = text.ToString();
                return true; //返回值
            }));
        }

        private void AlternationUsingFuncThread()
        {
            WaitCallback waitCallBack = new WaitCallback(this.AlternationUsingFunc);
            ThreadPool.QueueUserWorkItem(waitCallBack, "Func的使用");
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            AlternationUsingFuncThread();
        }
        #endregion

        #region 利用Action实现线程和界面交互
        private void AlternationUsingAction(object text)
        {
            //无参数,无返回值
            //button2.Dispatcher.Invoke(new Action(delegate()
            //{
            //    button2.Content = text.ToString();
            //}));
            //或者
            button2.Dispatcher.Invoke((Action)(()=>
            {
                button2.Content = text.ToString();
            }));
        }

        private void AlternationUsingActionThread()
        {
            WaitCallback waitCallBack = new WaitCallback(this.AlternationUsingAction);
            ThreadPool.QueueUserWorkItem(waitCallBack, "Action的使用");
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            AlternationUsingActionThread();
        }
        #endregion

        #region 利用Predicate实现线程和界面的交互
        private void AlternationUsingPrecidate(object text)
        {
            //需要一个T类型的参数,返回bool类型
            this.button3.Dispatcher.Invoke(new Predicate<object>(delegate(object myText)
            {
                myText = text;
                button3.Content = myText.ToString();
                return true;   //返回值
            }), text);
        }

        private void AlternationUsingPrecidateThread()
        {
            WaitCallback waitCallBack = new WaitCallback(this.AlternationUsingPrecidate);
            ThreadPool.QueueUserWorkItem(waitCallBack, "Predicate的使用");
        }

        private void button3_Click(object sender, RoutedEventArgs e)
        {
            AlternationUsingPrecidateThread();
        }
        #endregion

        
    }
}
