﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace DelegateIntegrateWinFormApp
{
    public partial class mainFrm : Form
    {
        public mainFrm()
        {
            InitializeComponent();
        }

        private void mainFrm_Load(object sender, EventArgs e)
        {
            /****************************注意例子中的使用方法****************
             * delegate TResult Func<TResult>();  无参,但是返回值为TResult类型
             * delegate void Action<T>(T1 arg1);  有一个参数arg1,但是无返回值
             * delegate bool Predicate<T>(T arg);  有一个参数arg,返回bool类型
             * **************************************************************/
        }

        #region 利用Func实现线程和界面交互
        private void AlternationUsingFunc(object text)
        {
            //无参数,但是返回值为bool类型
            this.Invoke(new Func<bool>(delegate()
            {
                button1.Text = text.ToString();
                return true; //返回值
            }));
        }

        private void AlternationUsingFuncThread()
        {
            WaitCallback waitCallBack = new WaitCallback(this.AlternationUsingFunc);
            ThreadPool.QueueUserWorkItem(waitCallBack, "Func的使用");
        }

        private void button1_Click(object sender, EventArgs e)
        {
            AlternationUsingFuncThread();
        }
        #endregion

        #region 利用Action实现线程和界面交互
        private void AlternationUsingAction(object text)
        {
            //需要一个T类型的参数,无返回值
            this.Invoke(new Action<object>(delegate(object myText)
            {
                myText = text;
                button2.Text = text.ToString();
            }),text);
        }

        private void AlternationUsingActionThread()
        {
            WaitCallback waitCallBack = new WaitCallback(this.AlternationUsingAction);
            ThreadPool.QueueUserWorkItem(waitCallBack,"Action的使用");
        }

        private void button2_Click(object sender, EventArgs e)
        {
            AlternationUsingActionThread();
        }
        #endregion

        #region 利用Predicate实现线程和界面的交互
        private void AlternationUsingPrecidate(object text)
        {
            //需要一个T类型的参数,返回bool类型
            this.Invoke(new Predicate<object>(delegate(object myText)  
            {
                myText = text;
                button3.Text = myText.ToString();
                return true;   //返回值
            }),text);
        }

        private void AlternationUsingPrecidateThread()
        {
            WaitCallback waitCallBack = new WaitCallback(this.AlternationUsingPrecidate);
            ThreadPool.QueueUserWorkItem(waitCallBack,"Predicate的使用");
        }

        private void button3_Click(object sender, EventArgs e)
        {
            AlternationUsingPrecidateThread();
        }
        #endregion

        


    }
}
